Package: triplex
Type: Package
Title: Search and visualize intramolecular triplex-forming sequences in
        DNA
Version: 1.51.0
Date: 2013-09-28
Authors@R: c(person("Jiri", "Hon", role = c("aut", "cre"),
                    email = "jiri.hon@gmail.com"),
             person("Matej", "Lexa", role = "aut",
                    email = "lexa@fi.muni.cz"),
             person("Tomas", "Martinek", role = "aut",
                    email = "martinto@fit.vutbr.cz"),
             person("Kamil", "Rajdl", role = "aut"),
             person("Daniel", "Kopecek", role = "ctb"))
Author: Jiri Hon, Matej Lexa, Tomas Martinek and Kamil Rajdl with contributions from Daniel Kopecek
Maintainer: Jiri Hon <jiri.hon@gmail.com>
Description: This package provides functions for identification and 
  visualization of potential intramolecular triplex patterns in DNA sequence.
  The main functionality is to detect the positions of subsequences capable of 
  folding into an intramolecular triplex (H-DNA) in a much larger sequence. 
  The potential H-DNA (triplexes) should be made of as many cannonical 
  nucleotide triplets as possible. The package includes visualization showing 
  the exact base-pairing in 1D, 2D or 3D.
License: BSD_2_clause + file LICENSE
URL: http://www.fi.muni.cz/~lexa/triplex/
biocViews: SequenceMatching, GeneRegulation
Depends: R (>= 2.15.0), S4Vectors (>= 0.5.14), IRanges (>= 2.5.27),
        XVector (>= 0.11.6), Biostrings (>= 2.39.10)
Imports: methods, grid, GenomicRanges
Suggests: rgl (>= 0.93.932), BSgenome.Celegans.UCSC.ce10, rtracklayer
LinkingTo: S4Vectors, IRanges, XVector, Biostrings
git_url: https://git.bioconductor.org/packages/triplex
git_branch: devel
git_last_commit: 53b853d
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-11-01 00:43:18 UTC; biocbuild
